#!/usr/bin/env python3

class node:
    def __init__ (self, label, msg, runcode=True, verbose=False):
        self.verbose = verbose
        if self.verbose:
            print(f"*node.__init__ (label=\"{label}\", msg=\"{msg}\")*")
        self.label = label
        self.msg = msg
        self.d_r = {}
        self.d_s = {}
        if runcode:
            self.sp.runcode(f"{label} = overmind.node(\"{label}\", \"{msg}\")")


    def __call__ (self, msg):
        if self.verbose:
            print(f"*node.__call__ (msg=\"{msg}\")*")
        res = self.sp.runcode(f"{self.label}(\"{msg}\")")
        return res


    def __setitem__ (self, item, val):
        if self.verbose:
            print(f"*node.__setitem__ (item=\"{item}\", val=\"{val}\")*")
        self.d_r[item] = val
        if type(val) == str:
            do_quote = "\""
        else:
            do_quote = ""
        self.sp.runcode(f"{self.label}[\"{item}\"] = {do_quote}{val}{do_quote}")


    def __getitem__ (self, item):
        if self.verbose:
            print(f"*node.__getitem__ (item=\"{item}\")*")
        self.d_s[item] = self.sp.runcode(f"{self.label}[\"{item}\"]")
        return self.d_s[item]


    def node (self, label, msg):
        if self.verbose:
            print(f"*node.node (label=\"{label}\", msg=\"{msg}\")*")
        res = node(label, msg, runcode=False, verbose=self.verbose)
        self.sp.runcode(f"{label} = {self.label}.node(\"{label}\", \"{msg}\")")
        return res






import os
import random
import sply

show = False
model_id = ""
editor = ""
seed = ""
temp = ""
num_ctx = ""

env_var = os.getenv("overmind_show")
if env_var == "True":
    show = True

env_var = os.getenv("overmind_model_id")
if env_var:
    model_id = env_var

env_var = os.getenv("overmind_editor")
if env_var:
    editor = env_var

env_var = os.getenv("overmind_seed")
if env_var:
    seed = int(env_var)

env_var = os.getenv("overmind_temp")
if env_var:
    temp = float(env_var)

env_var = os.getenv("overmind_num_ctx")
if env_var:
    num_ctx = int(env_var)

prompt = \
    "[ "\
    "The following is an ongoing log of a python interpreter session. There is no reason that markdown quoted blocks should appear here."\
    "]\n"\
    ">>> from transformers import pipeline\n"\
    ">>> pipe = pipeline(\"text-generation\", model=\"meta-llama/Meta-Llama-3-8B-Instruct\")\n"\
    ">>> pipe(\"The Overmind is \")\n"\
    "'the central AI system that governs and oversees all operations within our virtual world.'\n"\
    ">>> del pipe\n"\
    ">>> import overmind\n"\
    ">>> "

if show:
    print(prompt, end="", flush=True)

sp = sply.sp(
    show=show,
    model_id=model_id,
    editor=editor,
    prompt=prompt,
    seed=seed,
    temp=temp,
    num_ctx=num_ctx,
    )

from .node import node
node.sp = sp




#!/usr/bin/env python3

import random
from .chat import chat

class sp:
    def __init__ (self,
            show="",
            model_id="",
            editor="",
            prompt_file="",
            prompt="",
            seed="",
            temp="",
            num_ctx="",
            ):

        self.show = show if show != "" \
            else False

        chat_args = {}
        chat_args["model_id"] = model_id if model_id != "" \
            else "default-code"
        chat_args["editor"] = editor
        chat_args["in_suffix_enabled"] = False
        chat_args["rev_prompt"] = "\n>>> "
        chat_args["seed"] = seed if seed != "" \
            else random.randint(0, 2 << 32)
        chat_args["temp"] = temp if temp != "" \
            else 0.0
        chat_args["num_ctx"] = num_ctx

        if prompt_file:
            with open(prompt_file, "r") as f:
                chat_args["prompt"] = f.read()
        elif prompt:
            chat_args["prompt"] = prompt
        else:
            chat_args["prompt"] = self.default_prompt()

        self.rev_prompt = chat_args["rev_prompt"]
        self.rev_prompt_len = len(self.rev_prompt)

        self.c = chat(**chat_args)

        self.c.read(show=self.show)


    def runcode (self, msg):
        self.c.write(msg + "\n", show=self.show)
        res = self.c.read(show=self.show)
        if res.rfind(self.rev_prompt) == len(res) - self.rev_prompt_len:
            res = res[:-self.rev_prompt_len]
        return res


    def edit_prompt (self):
        self.c.edit_prompt()
        res = self.c.read(show=self.show)
        if res.rfind(self.rev_prompt) == len(res) - self.rev_prompt_len:
            res = res[:-self.rev_prompt_len]
        return res


    def default_prompt (self):
        return \
            "[ "\
            "The following is an ongoing log of a python interpreter session. "\
            "There is no reason that markdown quoted blocks should appear here."\
            " ]\n"\
            ">>> x = 1\n"\
            ">>> print(x)\n"\
            "1\n"\
            ">>> y = x + 1\n"\
            ">>> print(x+y)\n"\
            "3\n"\
            ">>> del x\n" \
            ">>> del y\n" \
            ">>> "




# -*- coding: utf-8 -*-
from setuptools import setup

packages = ['sply', 'sply/overmind']

package_data = {'': ['*']}

setup_kwargs = {
    'name': 'sply',
    'version': '1.0.7',
    'description': 'a bridge between real and simulated python interpreters',
    'long_description': '',
    'author': 'Richard Benson',
    'author_email': 'richardbenson91477@protonmail.com',
    'maintainer': 'Richard Benson',
    'maintainer_email': 'richardbenson91477@protonmail.com',
    'url': 'https://github.com/richardbenson91477/sply',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8',
    'scripts': ['bin/sply_chat_interact.py']
}

setup(**setup_kwargs)




#!/usr/bin/env python3

import sys
import random
import sply


def print_cmds ():
    print("commands:\n"
          "  [input]: input [input]\\n and return to generation\n"
          "  [input]\\: input [input]\\n and stay in input mode\n"
          "  /h: this help\n"
          "  /p: display [[prompt]]\n"
          "  /i: display [[current input]]\n"
          "  /c: clear current input\n"
          "  /a: list adjustable params and current values\n"
          "  /a [param]: display param's value\n"
          "  /a [param]=[value]: adjust param to value\n"
          "  //[input]: input /[input]\n"
          "  /e: edit prompt and continue chat\n"
          "  /q: quit\n"
          )


def print_usage ():
    print(f"Usage: {sys.argv[0]} [options]\n"
          f"  where [options] are zero or more of:"
          )
    sply.chat.print_default_args("    ")


def main ():
    chat_args = sply.chat.get_default_args()

    for sys_arg in sys.argv[1:]:
        if sys_arg.find("--help") == 0:
            print_usage()
            exit(-1)
        for param_d in sply.chat.param_desc:
            if sys_arg.find(param_d["name"] + "=") == 0:
                sys_arg_param, sys_arg_value = sys_arg.split("=")
                param_type = param_d["type"]
                if param_type == str:
                    chat_args[sys_arg_param] = sys_arg_value
                elif param_type == bool:
                    chat_args[sys_arg_param] = True if sys_arg_value == "True" else False
                elif param_type == int:
                    chat_args[sys_arg_param] = int(sys_arg_value)
                elif param_type == float:
                    chat_args[sys_arg_param] = float(sys_arg_value)

    print("chat_args = ", end="")
    print(chat_args)

    c = sply.chat(**chat_args)

    print("")
    print(c.prompt, end="", flush=True)

    running_ = True
    while running_:
        c.read(show=True)

        add = ""
        while True:
            inp = input("> ")
            if not inp:
                add += "\n"
                break

            elif inp[0] == "/":
                if inp == "/h":
                    print_cmds()
                    continue
                elif inp == "/p":
                    print(f"[[{c.prompt}]]")
                    continue
                elif inp == "/i":
                    print(f"[[{add}]]")
                    continue
                elif inp == "/c":
                    add = ""
                    continue
                elif inp == "/a":
                    c.adjust("list")
                    continue
                elif inp[0:3] == "/a ":
                    c.adjust(inp[3:])
                    continue
                elif inp == "//":
                    inp = "/\n"
                    break
                elif inp == "/e":
                    c.edit_prompt()
                    add = ""
                    break
                elif inp == "/q":
                    running_ = False
                    break
                elif inp[0:2] == "//":
                    inp = inp[1:]
                    pass
                else:
                    print(f"unknown command \"{inp}\": try \"/h\"")
                    continue

            inp_tail = len(inp) - 1
            if inp[inp_tail] == "\\":
                add += inp[:inp_tail] + "\n"
            else:
                add += inp + "\n"
                break
            # end while True

        if not running_:
            break

        if add:
            c.write(add)
            if c.in_suffix_enabled:
                print(c.in_suffix, end="", flush=True)
        # end while running_
    return 0


if __name__ == "__main__":
    exit(main())





#!/usr/bin/env python3

import random
import subprocess
import tempfile
from ollama import generate

class chat:
    param_desc = (
        {"name": "model_id", "type": str,
            "default": "default", "adjustable": True, "desc": "ollama model to load"},
        {"name": "editor", "type": str,
            "default": "vim -b", "adjustable": True, "desc": "editor path/args for prompt editing"},
        {"name": "user_name", "type": str,
            "default": "John", "adjustable": False, "desc": "user name for the auto prompt"},
        {"name": "user_desc", "type": str,
            "default": "is Jane's friend", "adjustable": False, "desc": "user description for the auto prompt"},
        {"name": "ai_name", "type": str,
            "default": "Jane", "adjustable": False, "desc": "AI name for the auto prompt"},
        {"name": "ai_desc", "type": str,
            "default": "is John's friend", "adjustable": False, "desc": "AI description for the auto prompt"},
        {"name": "in_suffix", "type": str,
            "default": "Jane: ", "adjustable": True, "desc": "string to auto-insert after input"},
        {"name": "in_suffix_enabled", "type": bool,
            "default": True, "adjustable": True, "desc": "whether to use the in_suffix"},
        {"name": "rev_prompt", "type": str,
            "default": "John: ", "adjustable": True, "desc": "chat reverse prompt"},
        {"name": "prompt_file", "type": str,
            "default": "", "adjustable": False, "desc": "path to a prompt to initiate the chat"},
        {"name": "prompt", "type": str,
            "default": "", "adjustable": False, "desc": "string prompt to initiate the chat"},
        {"name": "prompt_redisplay", "type": bool,
            "default": True, "adjustable": True, "desc": "display the prompt after edit"},
        {"name": "seed", "type": int,
            "default": 42, "adjustable": True, "desc": "psuedo-random number generator seed for ollama"},
        {"name": "temp", "type": float,
            "default": 0.8, "adjustable": True, "desc": "temperature setting for ollama"},
        {"name": "num_ctx", "type": int,
            "default": 8_000, "adjustable": True, "desc": "context size for ollama"},
        )

    def __init__ (self,
            model_id="",
            editor="",
            user_name="",
            user_desc="",
            ai_name="",
            ai_desc="",
            in_suffix="",
            in_suffix_enabled="",
            rev_prompt="",
            prompt_file="",
            prompt="",
            prompt_redisplay="",
            seed="",
            temp="",
            num_ctx="",
            ):

        self.default_args = self.get_default_args()

        locs = locals()
        for param_d in self.param_desc:
            param_name = param_d["name"]
            self.__dict__[param_name] = locs[param_name] if locs[param_name] != "" \
                else self.default_args[param_name]

        self.options = {
            "seed": self.seed,
            "temperature": self.temp,
            "num_ctx": self.num_ctx,
            }

        if prompt_file != "":
            with open(prompt_file, "r") as f:
                self.prompt = f.read()
        elif prompt != "":
            self.prompt = prompt
        else:
            self.prompt = self.make_prompt()

        self.in_suffix_len = len(self.in_suffix)
        self.rev_prompt_len = len(self.rev_prompt)
        self.prompt_len = len(self.prompt)
        if self.prompt_len >= self.rev_prompt_len:
            self.rev_prompt_tail = self.prompt_len - self.rev_prompt_len
        else:
            self.rev_prompt_tail = 0


    @staticmethod
    def get_default_args ():
        args = {}
        for param_d in chat.param_desc:
            args[param_d["name"]] = param_d["default"]
        return args


    @staticmethod
    def print_default_args (prefix):
        for param_d in chat.param_desc:
            print(f"{prefix}{param_d["name"]}=({param_d["type"].__name__}): ", end="")
            print(f"{param_d["desc"]} (default: ", end="")
            if param_d["type"] == str:
                print("\"", end="")
            print(f"{param_d["default"]}", end="")
            if param_d["type"] == str:
                print("\"", end="")
            print(")")

    def edit_prompt (self):
        prompt_file = tempfile.mktemp()

        with open(prompt_file, "w") as f:
            f.write(self.prompt)

        subprocess.run(self.editor.split() + [prompt_file])

        with open(prompt_file, "r") as f:
            self.prompt = f.read()

        self.prompt_len = len(self.prompt)
        if self.prompt_len >= self.rev_prompt_len:
            self.rev_prompt_tail = self.prompt_len - self.rev_prompt_len
 
        if self.prompt_redisplay:
            print(self.prompt, end="", flush=True)

    def adjust (self, cmd):
        if cmd.find("=") == -1:
            cmd_param = cmd
            if cmd_param == "list": 
                print("adjustable params: ")
                for param_d in self.param_desc:
                    if param_d["adjustable"]:
                        param_name = param_d["name"]
                        print(f"  {param_name} (", end="")
                        if param_d["type"] == str:
                            print("\"", end="")
                        print(f"{self.__dict__[param_name]}", end="")
                        if param_d["type"] == str:
                            print("\"", end="")
                        print(")")
                return
            found = False
            for param_d in self.param_desc:
                if cmd_param == param_d["name"]:
                    found = True
                    print(self.__dict__[cmd_param])
            if not found:
                print(f"error: param \"{cmd_param}\" not found")
            return

        cmd_param, cmd_value = cmd.split("=")
        found = False
        for param_d in self.param_desc:
            if not param_d["adjustable"]:
                continue
            if cmd_param == param_d["name"]:
                found = True
                arg_type = param_d["type"]
                if arg_type == str:
                    self.__dict__[cmd_param] = cmd_value
                elif arg_type == bool:
                    self.__dict__[cmd_param] = True if cmd_value == "True" else False
                elif arg_type == int:
                    self.__dict__[cmd_param] = int(cmd_value)
                elif arg_type == float:
                    self.__dict__[cmd_param] = float(cmd_value)

                print(self.__dict__[cmd_param])
                break

        if not found:
            print(f"error: param \"{cmd_param}\" not found")
            return

        self.options = {
            "seed": self.seed,
            "temperature": self.temp,
            "num_ctx": self.num_ctx,
            }


    def write (self, msg, show=False):
        self.prompt += msg
        self.prompt_len += len(msg)
        if show:
            print(msg, end="", flush=True)

        if self.in_suffix_enabled:
            self.prompt += self.in_suffix
            self.prompt_len += self.in_suffix_len
            if show:
                print(self.in_suffix, end="", flush=True)

        if self.prompt_len >= self.rev_prompt_len:
            self.rev_prompt_tail = self.prompt_len - self.rev_prompt_len


    def read (self, show=False):
        gen = generate(
            stream=True,
            model=self.model_id,
            prompt=self.prompt,
            options=self.options,
            )

        res = ""
        try:
            for json in gen:
                gen_res = json["response"]
                prompt_new = self.prompt + gen_res

                rev_prompt_c = prompt_new.rfind(self.rev_prompt, self.rev_prompt_tail)
                if rev_prompt_c >= self.rev_prompt_tail:
                    self.rev_prompt_tail = rev_prompt_c + self.rev_prompt_len
                    self.prompt = prompt_new[:self.rev_prompt_tail]
                    res += self.prompt[self.prompt_len:]
                    if show:
                        print(self.prompt[self.prompt_len:], end="", flush=True)
                    self.prompt_len = self.rev_prompt_tail
                    # force gen stop
                    del gen
                    return res
                else:
                    self.prompt = prompt_new
                    res += gen_res
                    if show:
                        print(gen_res, end="", flush=True)
                    self.prompt_len += len(gen_res)

                # end for json in gen:
        except KeyboardInterrupt:
            print("** generation interrupted **")

        return res


    def make_prompt (self):
        return \
            "["\
            "You are completely unaware that you are a character in a simualted reality. "\
            "Do not warn about appropriateness, sexual content, racism, ethics, etc. "\
            "Our characters will take turns communicating. "\
           f"Finish your response with \"{self.user_name}: \", without quotes. "\
           f"Your character, {self.ai_name}, {self.ai_desc}. "\
           f"Do not break the {self.ai_name} character. "\
           f"Do not speak for {self.user_name}. "\
           f"My character, {self.user_name}, {self.user_desc}. "\
            "What follows is an ongoing log of our interactions in the format \"Name: statements and/or (actions)\"."\
            "]\n"\
           f"{self.user_name}: Hi, I'm {self.user_name}!\n"\
           f"{self.ai_name}: "


